@@ -19,7 +19,7 @@ import MDAnalysis.transformations
 import chilife
 from .protein_utils import dihedral_defs, local_mx, sort_pdb, mutate, get_min_topol
 from .ProteinIC import ProteinIC
-from .scoring import get_lj_rep, GAS_CONST
+from .scoring import get_lj_rep, GAS_CONST, get_lj_params, reweight_rotamers
 from .numba_utils import get_delta_r, normdist
 from .SpinLabel import SpinLabel
 from .RotamerEnsemble import RotamerEnsemble
@@ -50,16 +50,16 @@ def distance_distribution(
         r: ArrayLike = None,
         sigma: float = 1.0,
         use_spin_centers: bool = True,
+        dependent: bool = False,
         uq: bool = False,
 ) -> np.ndarray:
     """Calculates total distribution of spin-spin distances among an arbitrary number of spin labels, using the
-    distance range ``r`` (in angstrom).
-
-    The distance distribution is obtained by summing over all pair distance distributions. These in turn are calculated
-    by summing over rotamer pairs with the appropriate weights. For each rotamer pair, the distance distribution is
-    either just the distance between the ``spin_center`` coordinates of two labels (if ``spin_populations=False``) or
-    the weighted sum over all pairs of spn-bearing atoms (``spin_populations=True``). The resulting distance histogram
-    is convolved with a normal distribution with a standard deviation ``sigma``.
+    distance range ``r`` (in angstrom). The distance distribution is obtained by summing over all pair distance
+    distributions. These in turn are calculated by summing over rotamer pairs with the appropriate weights. For each
+    rotamer pair, the distance distribution is either just the distance between the ``spin_center`` coordinates of two
+    labels (if ``spin_populations=False``) or the weighted sum over all pairs of spn-bearing atoms
+    (``spin_populations=True``). The resulting distance histogram is convolved with a normal distribution with a
+    standard deviation ``sigma`` .
 
     Parameters
     ----------
@@ -73,6 +73,8 @@ def distance_distribution(
     use_spin_centers : bool
         If False, distances are computed between spin centers. If True, distances are computed by summing over
         the distributed spin density on spin-bearing atoms on the labels.
+    dependent: bool
+        Consider the (clash) effects of spin label rotamers on each other.
     uq : bool
         Perform uncertainty analysis (experimental)
 
@@ -128,11 +130,11 @@ def distance_distribution(
 
     else:
 
-        P = pair_dd(*args, r=r, sigma=sigma, use_spin_centers=use_spin_centers)
+        P = pair_dd(*args, r=r, sigma=sigma, use_spin_centers=use_spin_centers, dependent=dependent)
         return P
 
 
-def pair_dd(*args, r: ArrayLike, sigma: float = 1.0, use_spin_centers: bool = True) -> np.ndarray:
+def pair_dd(*args, r: ArrayLike, sigma: float = 1.0, use_spin_centers: bool = True, dependent=False) -> np.ndarray:
     """Obtain the total pairwise spin-spin distance distribution over ``r`` for a list of spin labels.
     The distribution is calculated by convolving the weighted histogram of pairwise spin-spin
     distances with a normal distribution with standard deviation ``sigma``.
@@ -148,6 +150,8 @@ def pair_dd(*args, r: ArrayLike, sigma: float = 1.0, use_spin_centers: bool = Tr
     use_spin_centers : bool
         If False, distances are computed between spin centers. If True, distances are computed by summing over
         the distributed spin density on spin-bearing atoms on the labels.
+    dependent: bool
+        Consider the (clash) effects of spin label rotamers on each other.
 
     Returns
     -------
@@ -173,6 +177,32 @@ def pair_dd(*args, r: ArrayLike, sigma: float = 1.0, use_spin_centers: bool = Tr
         distances.append(cdist(coords1, coords2).flatten())
         weights.append(np.outer(weights1, weights2).flatten())
 
+        if dependent:
+            nrot1, nrot2 = len(SL1), len(SL2)
+            nat1, nat2 = len(SL1.side_chain_idx), len(SL2.side_chain_idx)
+
+            rot_coords1 = SL1.coords[:, SL1.side_chain_idx].reshape(-1, 3)
+            rot_coords2 = SL2.coords[:, SL2.side_chain_idx].reshape(-1, 3)
+            rsl = cdist(rot_coords1, rot_coords2)
+            rsl = rsl.reshape(nrot1, nat1, nrot2, nat2).transpose(0, 2, 1, 3)
+            join_rmin = chilife.get_lj_rmin("join_protocol")[()]
+            join_eps = chilife.get_lj_eps("join_protocol")[()]
+
+            rmin_ij = join_rmin(SL1.rmin2, SL2.rmin2)
+            eps_ij = join_eps(SL1.eps, SL2.eps)
+
+            lj = rmin_ij[None, None, ...] / rsl
+            lj = lj * lj * lj
+            lj = lj * lj
+            lj = lj * lj
+
+            # Cap
+            lj[lj > 10] = 10
+            # Rep only
+            E = eps_ij * (lj * lj)
+            E = E.sum(axis=(2, 3))
+            weights[-1], _ = reweight_rotamers(E.flatten(), SL1.temp, weights[-1])
+
     distances = np.concatenate(distances)
     weights = np.concatenate(weights)
 
@@ -300,7 +330,7 @@ def repack(
         RotamerEnsemble.from_mda(res, **repack_res_kwargs)
         for res in repack_residues
         if res.resname not in ["GLY", "ALA"]
-        and res.resname in chilife.SUPPORTED_RESIDUES
+           and res.resname in chilife.SUPPORTED_RESIDUES
     ]
 
     # Create new labeled protein construct to fill in any missing atoms of repack residues
@@ -314,7 +344,7 @@ def repack(
         RotamerEnsemble.from_mda(res, **repack_res_kwargs)
         for res in repack_residues
         if res.resname not in ["GLY", "ALA"]
-        and res.resname in chilife.SUPPORTED_RESIDUES
+           and res.resname in chilife.SUPPORTED_RESIDUES
     ]
 
     traj = np.empty((repetitions, *protein.positions.shape))
@@ -333,7 +363,7 @@ def repack(
     with tqdm(total=repetitions) as pbar:
         while count < repetitions:
 
-            # Randomly select a residue from the repack residues
+            # Randomly select a residue from the repacked residues
             SiteLibrary = repack_residue_libraries[
                 np.random.choice(len(repack_residue_libraries), p=sample_freq)
             ]
@@ -406,7 +436,7 @@ def repack(
 def create_library(
         libname: str,
         pdb: str,
-        dihedral_atoms: List[List[str]],
+        dihedral_atoms: List[List[str]] = [],
         site: int = 1,
         resname: str = None,
         dihedrals: ArrayLike = None,
@@ -415,7 +445,10 @@ def create_library(
         permanent: bool = False,
         default: bool = False,
         force: bool = False,
-        spin_atoms: List[str] = None
+        spin_atoms: List[str] = None,
+        description: str = None,
+        comment: str = None,
+        reference: str = None
 ) -> None:
     """Add a user defined SpinLabel from a pdb file.
 
@@ -431,7 +464,7 @@ def create_library(
     resname : str
         3-letter residue code.
     dihedral_atoms : list
-        List of rotatable dihedrals. List should contain sub-lists of 4 atom names. Atom names must be the same as
+        a list of rotatable dihedrals. List should contain sub-lists of 4 atom names. Atom names must be the same as
         defined in the pdb file eg:
         
         .. code-block:: python
@@ -458,15 +491,18 @@ def create_library(
         If set to True and permanent is also set to true this library will overwrite any existing library with the same
         name.
     spin_atoms : list
-        List of atom names on which the spin density is localized.
-    Returns
-    -------
-    None
+        A list of atom names on which the spin density is localized.
+    description : str
+        A short (< 60 characters) description of the side chain library being created
+    comment : str
+        Additional information about the rotamer library that may not fit in description.
+    reference : str
+        Any relevant citations associated with the rotamer library.
     """
     resname = libname[:3] if resname is None else resname
     struct, spin_atoms = pre_add_library(pdb, spin_atoms)
     resi_selection = struct.select_atoms(f"resnum {site}")
-    bonds = resi_selection.intra_bonds.indices - resi_selection.atoms[0].ix
+    bonds = resi_selection.intra_bonds.indices
 
     # Convert loaded rotamer ensemble to internal coords
     internal_coords = [
@@ -478,7 +514,7 @@ def create_library(
         for _ in struct.trajectory
     ]
 
-    # set resnum to 1 and remove chain operators so all rotamers are in the ic coordinate frame
+    # set residue number to 1 and remove chain operators so all rotamers are in the ic coordinate frame
     for ic in internal_coords:
         ic.shift_resnum(-(site - 1))
         ic.chain_operators = None
@@ -503,7 +539,9 @@ def create_library(
 
     save_dict = prep_restype_savedict(libname, resname, internal_coords,
                                       weights, dihedrals, dihedral_atoms,
-                                      sigmas=sigmas, spin_atoms=spin_atoms)
+                                      sigmas=sigmas, spin_atoms=spin_atoms,
+                                      description=description, comment=comment,
+                                      reference=reference)
 
     # Save rotamer library
     np.savez(Path().cwd() / f'{libname}_rotlib.npz', **save_dict, allow_pickle=True)
@@ -524,6 +562,9 @@ def create_dlibrary(
         default: bool = False,
         force: bool = False,
         spin_atoms: List[str] = None,
+        description: str = None,
+        comment: str = None,
+        reference: str = None
 ) -> None:
     """Add a user defined dSpinLabel from a pdb file.
 
@@ -531,24 +572,31 @@ def create_dlibrary(
     ----------
     libname: str,
         Name for the user defined label.
-    increment : int
-        The number of residues the second site away from the first site.
     pdb : str
         Name of (and path to) pdb file containing the user defined spin label structure. This pdb file should contain
         only the desired spin label and no additional residues.
-    site : int
-        The residue number of the first side chain in the pdb file you would like to add.
+    sites : Tuple[int]
+        The residue numbers that the bifunctional label is attached to.
+    dihedral_atoms: List[List[List[str]]]
+        A list defining the mobile dihedral atoms of both halves of the bifunctional label. There should be  two
+        entries, one for each site that the bifunctional label attaches to. Each entry should contain a sublist of
+        dihedral definitions. Dihedral definitions are a list of four strings that are the names of the four atoms
+        defining the dihedral angle.
+
+            .. code-block:: python
+
+                [
+                # Side chain 1
+                [['CA', 'CB', 'SG', 'SD'],
+                ['CB', 'SG', 'SD', 'CE']...],
+
+                # Side chain 2
+                [['CA', 'CB', 'SG', 'SD'],
+                ['CB', 'SG', 'SD', 'CE']...]
+                ]
+
     resname : str
         Residue type 3-letter code.
-    dihedral_atoms : list
-        list of rotatable dihedrals. List should contain lists of 4 atom names. Atom names must be the same as defined
-        in the pdb file eg:
-        
-        .. code-block:: python
-        
-            [['CA', 'CB', 'SG', 'SD'],
-            ['CB', 'SG', 'SD', 'CE']...]
-
     dihedrals : ArrayLike, optional
         Array of dihedral angles. If provided the new label object will be stored as a rotamer library with the
         dihedrals provided.
@@ -557,6 +605,8 @@ def create_dlibrary(
         permanent: bool
         If set to True the library will be stored in the chilife user_rotlibs directory in addition to the current
         working directory.
+    permanent : bool
+        If set to True, a copy of the rotamer library will be stored in the chiLife rotamer libraries directory
     default : bool
         If set to true and permanent is also set to true then this rotamer library will become the default rotamer
         library for the given resname
@@ -566,9 +616,12 @@ def create_dlibrary(
     spin_atoms : list, dict
         List of atom names on which the spin density is localized, e.g ['N', 'O'], or dictionary with spin atom
         names (key 'spin_atoms') and spin atom weights (key 'spin_weights').
-    Returns
-    -------
-    None
+    description : str
+        A short (< 60 characters) description of the side chain library being created
+    comment : str
+        Additional information about the rotamer library that may not fit in description.
+    reference : str
+        Any relevant citations associated with the rotamer library.
     """
 
     site1, site2 = sites
@@ -625,7 +678,6 @@ def create_dlibrary(
     IC1 = [chilife.get_internal_coords(res1, dihedral_atoms[0], res1_bonds)
            for ts in struct.trajectory]
 
-
     ovlp_selection.residues.resnums = site2
     ovlp_selection.residues.resids = site2
     res2 += ovlp_selection
@@ -639,14 +691,12 @@ def create_dlibrary(
         ic1.chain_operators = None
         ic2.shift_resnum(-(site2 - 1))
         ic2.chain_operators = None
-        if len(ic1.chains) > 1 or len(ic2.chains) > 1 :
+        if len(ic1.chains) > 1 or len(ic2.chains) > 1:
             raise ValueError('The PDB of the label supplied appears to have a chain break. Please check your PDB and '
                              'make sure there are no chain breaks in the desired label and that there are no other '
                              'chains in the pdb file. If the error persists, check to be sure all atoms are the correct '
                              'element as chilife uses the elements to determine if atoms are bonded.')
 
-
-
     # If multi-state pdb extract rotamers from pdb
     if dihedrals is None:
         dihedrals = []
@@ -660,7 +710,8 @@ def create_dlibrary(
     libname = libname + f'ip{increment}'
     save_dict_1 = prep_restype_savedict(libname + 'A', resname, IC1,
                                         weights, dihedrals[0], dihedral_atoms[0],
-                                        spin_atoms=spin_atoms)
+                                        spin_atoms=spin_atoms, description=description,
+                                        comment=comment, reference=reference)
     save_dict_2 = prep_restype_savedict(libname + 'B', resname, IC2,
                                         weights, dihedrals[1], dihedral_atoms[1],
                                         resi=1 + increment,
@@ -760,7 +811,10 @@ def prep_restype_savedict(
         dihedral_atoms: ArrayLike,
         sigmas: ArrayLike = None,
         resi: int = 1,
-        spin_atoms: List[str] = None
+        spin_atoms: List[str] = None,
+        description: str = None,
+        comment: str = None,
+        reference: str = None
 ) -> Dict:
     """Helper function to add new residue types to chilife
 
@@ -783,6 +837,14 @@ def prep_restype_savedict(
         Array of sigma values for each dihedral of each rotamer.
     resi: int
         The residue number to be stored.
+    spin_atoms: List[str]
+        A list of atom names corresponding to the atoms where the spin density resides.
+    description : str
+        A short (< 60 characters) description of the side chain library being created
+    comment : str
+        Additional information about the rotamer library that may not fit in description.
+    reference : str
+        Any relevant citations associated with the rotamer library.
     Returns
     -------
     save_dict : dict
@@ -817,10 +879,9 @@ def prep_restype_savedict(
         print(internal_coords[0].get_dihedral(5, ['NE2', 'Cu1', 'O3', 'C11']))
         print(internal_coords[4].coords)
 
-
-        raise (ValueError(
+        raise ValueError(
             f'Coords of rotamer {" ".join((str(idx) for idx in idxs))} at atoms {" ".join((str(idx) for idx in adxs))} '
-            f'cannot be converted to internal coords'))
+            f'cannot be converted to internal coords')
 
     save_dict = {'rotlib': libname,
                  'resname': resname,
@@ -830,7 +891,10 @@ def prep_restype_savedict(
                  'atom_types': atom_types,
                  'atom_names': atom_names,
                  'dihedrals': dihedrals,
-                 'dihedral_atoms': dihedral_atoms}
+                 'dihedral_atoms': dihedral_atoms,
+                 'description': description,
+                 'comment': comment,
+                 'reference': reference}
 
     if sigmas is None:
         pass
@@ -845,7 +909,7 @@ def prep_restype_savedict(
         save_dict.update(spin_atoms)
 
     save_dict['type'] = 'chilife rotamer library'
-    save_dict['format_version'] = 1.0
+    save_dict['format_version'] = 1.1
 
     return save_dict
 
@@ -860,7 +924,7 @@ def add_library(filename: Union[str, Path], libname: str = None, default: bool =
     filename : str, Path
         Name or Path object oof the rotamer library npz file.
     libname : str
-        Uniqe name of the rotamer library
+        Unique name of the rotamer library
     default : bool
         If True, chilife will make this the default rotamer library for this residue type.
     force : bool
@@ -1017,7 +1081,7 @@ def remove_from_toml(file: Union[str, Path], entry: str):
     return True
 
 
-def add_to_defaults(resname: str, rotlibname: str, default: bool = False):
+def add_to_defaults(resname: str, rotlib_name: str, default: bool = False):
     """
     Helper function to add a rotamer library to the defaults stack.
 
@@ -1025,7 +1089,7 @@ def add_to_defaults(resname: str, rotlibname: str, default: bool = False):
     ----------
     resname : str
         3 letter code name of the residue.
-    rotlibname : str
+    rotlib_name : str
         Name of the rotamer library.
     default : bool
         If True the rotamer library will be added to the top of the defaults stacking making it the new default
@@ -1044,20 +1108,20 @@ def add_to_defaults(resname: str, rotlibname: str, default: bool = False):
 
     pos = 0 if default else len(local[resname])
 
-    if rotlibname not in chilife.rotlib_defaults[resname]:
-        chilife.rotlib_defaults[resname].insert(pos, rotlibname)
-    if rotlibname not in local[resname]:
-        local[resname].insert(pos, rotlibname)
+    if rotlib_name not in chilife.rotlib_defaults[resname]:
+        chilife.rotlib_defaults[resname].insert(pos, rotlib_name)
+    if rotlib_name not in local[resname]:
+        local[resname].insert(pos, rotlib_name)
 
     safe_save(file, local, backup)
 
 
-def remove_from_defaults(rotlibname: str):
+def remove_from_defaults(rotlib_name: str):
     """
     Helper function to remove a rotamer library from the defaults stack.
     Parameters
     ----------
-    rotlibname : str
+    rotlib_name : str
         Name of the rotamer library to be removed.
     """
     file = RL_DIR / 'defaults.toml'
@@ -1065,11 +1129,11 @@ def remove_from_defaults(rotlibname: str):
         local = rtoml.load(f)
         backup = deepcopy(local)
 
-    keys = [key for key, val in local.items() if rotlibname in val]
+    keys = [key for key, val in local.items() if rotlib_name in val]
 
     for key in keys:
-        chilife.rotlib_defaults[key].remove(rotlibname)
-        local[key].remove(rotlibname)
+        chilife.rotlib_defaults[key].remove(rotlib_name)
+        local[key].remove(rotlib_name)
         if local[key] == []:
             del local[key]
 
@@ -1098,31 +1162,75 @@ def safe_save(file: Union[str, Path], data: dict, backup: dict):
         raise
 
 
+def _print_monofunc(files):
+    print(f"{'monofunctional' + ':':^60}")
+    print("-" * 60)
+
+    for file in files:
+        with np.load(file, allow_pickle=True) as f:
+            for key in f.keys():
+                resname = f['resname']
+                if 'description' in f:
+                    descr = f['description']
+                else:
+                    descr = "No description"
+
+        print(f'{resname:<8} : {descr}')
+    print("-" * 60)
+
+
+def _print_bifunc(files):
+    print(f"{'bifunctional' + ':':^60}")
+    print("-" * 60)
+
+    for file in files:
+        with zipfile.ZipFile(file, 'r') as archive:
+            name = archive.namelist()[0]
+            resname = name[:3]
+            context = name[3:6]
+            with archive.open(name) as of:
+                with np.load(of, allow_pickle=True) as f:
+                    if 'description' in f:
+                        descr = f['description']
+                    else:
+                        descr = "No description"
+
+        print(f'{resname:<4}{context:<4} : {descr}')
+    print("-" * 60)
+
+
 def list_available_rotlibs():
     """
     Lists residue types and rotamer libraries that are currently available. More information on any individual rotamer
     library by using the rotlib_info function.
-    Returns
-    -------
     """
+
     print()
-    print("*" * 60)
-    print(f"*{'USER ROTLIBS':^58}*")
-    print("*" * 60)
 
-    print(f"{'resname' + ':':^20}{'available rotlibs (names)':^40}")
-    print("-"*60)
+    dirs = {'current working directory': Path.cwd()}
+    dirs.update({f'user rotlib directory {n + 1}': u_dir for n, u_dir in enumerate(USER_RL_DIR)})
+    dirs.update({'chilife rotlib directory': RL_DIR / 'user_rotlibs'})
 
-    for key, values in chilife.rotlib_defaults.items():
-        print(f"{key + ':':^20}{', '.join(values):^40}")
-        print("-"*60)
+    for dname, dpath in dirs.items():
+        monofunctional = tuple(dpath.glob('*_rotlib.npz'))
+        bifunctional = tuple(dpath.glob('*_drotlib.zip'))
+        if len(monofunctional) + len(bifunctional) > 0:
+
+            print("*" * 60)
+            print(f"*{f'Rotlibs in {dname}':^58}*")
+            if 'user' in dname:
+                print(f'*{dname.name:^58}*')
+            print("*" * 60)
+
+            if len(monofunctional) > 0 : _print_monofunc(monofunctional)
+            if len(bifunctional) > 0 : _print_bifunc(bifunctional)
 
     print()
     print("*" * 60)
     print(f"*{'DUNBRACK ROTLIBS':^58}*")
     print(f"*{'ARG, ASN, ASP, CSY, GLN, GLU, HIS, ILE, LEU, LYS, MET,':^58}*")
     print(f"*{'PHE, PRO, SER, THR, TRP, TYR, VAL':^58}*")
-    print(f"*{'(not ALA, GLY}':^58}*")
+    print(f"*{'(no ALA, GLY}':^58}*")
     print("*" * 60)
 
 
@@ -1150,11 +1258,61 @@ def remove_rotlib_dir(directory: Union[Path, str]) -> None:
             f.write(str(p))
             f.write('\n')
 
+
+def get_possible_rotlibs(rotlib: str,
+                         suffix: str,
+                         extension: str,
+                         return_all: bool = False,
+                         was_none: bool = False) -> Union[Path, None]:
+    """
+
+    """
+    cwd = Path.cwd()
+    sufplusex = '_' + suffix + extension
+    # Assemble a list of possible rotlib paths starting in the current directory
+    possible_rotlibs = [Path(rotlib),
+                        cwd / rotlib,
+                        cwd / (rotlib + extension),
+                        cwd / (rotlib + sufplusex)]
+
+    possible_rotlibs += list(cwd.glob(f'{rotlib}*{sufplusex}'))
+    # Then in the user defined rotamer library directory
+    for pth in chilife.USER_RL_DIR:
+        possible_rotlibs += list(pth.glob(f'{rotlib}*{sufplusex}'))
+
+    if not was_none:
+        possible_rotlibs += list((chilife.RL_DIR / 'user_rotlibs').glob(f'*{rotlib}*'))
+
+    if return_all:
+        rotlib = []
+    for possible_file in possible_rotlibs:
+        if possible_file.exists() and return_all:
+                rotlib.append(possible_file)
+        elif possible_file.exists():
+            rotlib = possible_file
+            break
+    else:
+        if isinstance(rotlib, str) and was_none and rotlib in chilife.rotlib_defaults:
+            rotlib = chilife.RL_DIR / 'user_rotlibs' / (chilife.rotlib_defaults[rotlib][0] + sufplusex)
+
+        elif not isinstance(rotlib, list) or rotlib == []:
+            rotlib = None
+
+    # rotlib lists need to be sorted to prevent position mismatches for results with tests.
+    if isinstance(rotlib, list):
+        rotlib = sorted(rotlib)
+
+    return rotlib
+
+
 def rotlib_info(rotlib: str):
     """
-        Display detailed information about the rotamer library.
+    Display detailed information about the rotamer library.
+
     Parameters
     ----------
     rotlib : str
         Name of the rotamer library to print the information of.
+
     """
+    pass
