@@ -1,14 +1,16 @@
 import networkx as nx
-from joblib import Parallel, delayed
-from typing import Union
+from copy import deepcopy
 from itertools import combinations
 import logging
 import warnings
+import inspect
+from functools import partial
 from pathlib import Path
 
 import numpy as np
 from scipy.spatial import cKDTree
 import scipy.optimize as opt
+import MDAnalysis as mda
 
 import chilife
 
@@ -28,32 +30,34 @@ class dRotamerEnsemble:
         self.chain = chain if chain is not None else self.guess_chain()
         self.protein_tree = self.kwargs.setdefault("protein_tree", None)
 
-        self.forgive = kwargs.setdefault("forgive", 1.0)
-        self.clash_radius = kwargs.setdefault("clash_radius", 14.0)
+        self.forgive = kwargs.setdefault("forgive", 0.95)
         self._clash_ori_inp = kwargs.setdefault("clash_ori", "cen")
-        self.restraint_weight = kwargs.pop("restraint_weight") if "restraint_weight" in kwargs else 222  # kcal/mol/A^2
+        self.restraint_weight = kwargs.pop("restraint_weight") if "restraint_weight" in kwargs else 222
+        self.torsion_weight = kwargs.pop("torsion_weight") if "torsion_weight" in kwargs else 5
         self.alignment_method = kwargs.setdefault("alignment_method", "bisect".lower())
         self.dihedral_sigmas = kwargs.setdefault("dihedral_sigmas", 25)
         self._exclude_nb_interactions = kwargs.setdefault('exclude_nb_interactions', 3)
         self._minimize = kwargs.pop("minimize", True)
         self.min_method = kwargs.pop('min_method', 'L-BFGS-B')
+        self.trim_tol = kwargs.pop('trim_tol', 0.005)
+        self._do_trim = kwargs.pop('trim', True)
         self.eval_clash = kwargs.pop("eval_clash", True)
-        self.energy_func = kwargs.setdefault("energy_func", chilife.get_lj_rep)
+        self.energy_func = kwargs.setdefault("energy_func", chilife.get_lj_energy)
         self.temp = kwargs.setdefault("temp", 298)
         self.get_lib(rotlib)
         self.create_ensembles()
 
-        self.RL1.backbone_to_site()
-        self.RL2.backbone_to_site()
+        self.RE1.backbone_to_site()
+        self.RE2.backbone_to_site()
 
-        self.cst_idx1 = np.where(self.RL1.atom_names[None, :] == self.csts[:, None])[1]
-        self.cst_idx2 = np.where(self.RL2.atom_names[None, :] == self.csts[:, None])[1]
-        self.rl1mask = np.argwhere(~np.isin(self.RL1.atom_names, self.csts)).flatten()
-        self.rl2mask = np.argwhere(~np.isin(self.RL2.atom_names, self.csts)).flatten()
+        self.cst_idx1 = np.where(self.RE1.atom_names[None, :] == self.csts[:, None])[1]
+        self.cst_idx2 = np.where(self.RE2.atom_names[None, :] == self.csts[:, None])[1]
+        self.rl1mask = np.argwhere(~np.isin(self.RE1.atom_names, self.csts)).flatten()
+        self.rl2mask = np.argwhere(~np.isin(self.RE2.atom_names, self.csts)).flatten()
 
         self.name = self.res
         if self.site1 is not None:
-            self.name = f"{self.RL1.nataa}{self.site1}{self.RL1.nataa}{self.site2}{self.res}"
+            self.name = f"{self.RE1.nataa}{self.site1}-{self.RE2.nataa}{self.site2}{self.res}"
         if self.chain is not None:
             self.name += f"_{self.chain}"
 
@@ -64,10 +68,21 @@ class dRotamerEnsemble:
         self._graph = nx.Graph()
         self._graph.add_edges_from(self.bonds)
 
+        self.clash_radius = kwargs.setdefault("clash_radius", None)
+        if self.clash_radius is None:
+            self.clash_radius = np.linalg.norm(self.clash_ori - self.coords, axis=-1).max() + 5
+
+        self.rmin2 = chilife.get_lj_rmin(self.atom_types[self.side_chain_idx])
+        self.eps = chilife.get_lj_eps(self.atom_types[self.side_chain_idx])
+
         self.protein_setup()
-        self.sub_labels = (self.RL1, self.RL2)
+        self.sub_labels = (self.RE1, self.RE2)
 
     def protein_setup(self):
+        if isinstance(self.protein, (mda.AtomGroup, mda.Universe)):
+            if not hasattr(self.protein.universe._topology, "altLocs"):
+                self.protein.universe.add_TopologyAttr('altLocs', np.full(len(self.protein.universe.atoms), ""))
+
         self.protein = self.protein.select_atoms("not (byres name OH2 or resname HOH)")
         self.clash_ignore_idx = self.protein.select_atoms(
             f"resid {self.site1} {self.site2} and segid {self.chain}"
@@ -87,6 +102,8 @@ class dRotamerEnsemble:
         ]
 
         _, self.irmin_ij, self.ieps_ij, _ = chilife.prep_internal_clash(self)
+        _, self.ermin_ij, self.eeps_ij = chilife.prep_external_clash(self)
+
         self.aidx, self.bidx = [list(x) for x in zip(*self.non_bonded)]
 
         if self._minimize:
@@ -113,21 +130,30 @@ class dRotamerEnsemble:
         return chain
 
     def get_lib(self, rotlib):
-        if self.protein is not None:
-            # get site1 backbone information from protein structure
-            sel_txt = f"resnum {self.site1} and segid {self.chain}"
 
+        # If given a dictionary use that as the rotlib
+        if isinstance(rotlib, dict):
+            if tuple(sorted(rotlib.keys())) != ('csts', 'libA', 'libB'):
+                raise RuntimeError('Non-file dRotamerEnsemble rotlibs must be dictionaries consisting of exactle 3 '
+                                   'entries: `csts`, `libA` and `libB`')
+            self.csts = rotlib['csts']
+            self.libA, self.libB = rotlib['libA'], rotlib['libB']
+            self.kwargs["eval_clash"] = False
+            return None
 
         rotlib = self.res if rotlib is None else rotlib
         if 'ip' not in rotlib:
             rotlib += f'ip{self.increment}'
 
         # Check if any exist
-        rotlib_path = get_possible_rotlibs(rotlib)
+        rotlib_path = chilife.get_possible_rotlibs(rotlib, suffix='drotlib', extension='.zip')
 
         if rotlib_path is None:
             # Check if libraries exist but for different i+n
-            rotlib_path = get_possible_rotlibs(rotlib.replace(f'ip{self.increment}', ''), all=True)
+            rotlib_path = chilife.get_possible_rotlibs(rotlib.replace(f'ip{self.increment}', ''),
+                                                       suffix='drotlib',
+                                                       extension='.zip',
+                                                       return_all=True)
 
             if rotlib_path is None:
                 raise NameError(f'There is no rotamer library called {rotlib} in this directory or in chilife')
@@ -151,32 +177,29 @@ class dRotamerEnsemble:
 
             for lib in (libA, libB):
                 names, types = lib['atom_names'], libA['atom_types']
-                residxs =  np.zeros(len(names), dtype=int)
+                residxs = np.zeros(len(names), dtype=int)
                 resnames, resids = np.array([self.res]), np.array([1])
                 segidx = np.array([0])
 
                 uni = chilife.make_mda_uni(names, types, resnames, residxs, resids, segidx)
                 unis.append(uni)
 
-
             for p in rotlib_path:
                 tlibA, tlibB, tcsts = chilife.read_library(p)
                 for lib, tlib, cct, uni in zip((libA, libB), (tlibA, tlibB), (cctA, cctB), unis):
 
                     # Libraries must have the same atom order
                     if not np.all(np.isin(tlib['atom_names'], lib['atom_names'])) and \
-                           np.all(tlib['dihedral_atoms'] == lib['dihedral_atoms']):
-
+                            np.all(tlib['dihedral_atoms'] == lib['dihedral_atoms']):
                         raise ValueError(f'Rotlibs {rotlib_path[0].stem} and {p.stem} are not compatable. You may'
                                          f'need to rename one of them.')
 
-
                     # Map coordinates
                     ixmap = [np.argwhere(tlib['atom_names'] == aname).flat[0] for aname in lib['atom_names']]
                     cct.setdefault('coords', []).append(tlib['coords'][:, ixmap])
 
                     # Create new internal coords if they are defined differently
-                    lib_ic, tlib_ic =lib['internal_coords'][0], tlib['internal_coords'][0]
+                    lib_ic, tlib_ic = lib['internal_coords'][0], tlib['internal_coords'][0]
                     if np.any(lib_ic.atom_names != tlib_ic.atom_names):
                         uni.load_new(cct['coords'][-1])
                         ics = [chilife.get_internal_coords(uni, lib['dihedral_atoms'], lib_ic.bonded_pairs)
@@ -195,67 +218,64 @@ class dRotamerEnsemble:
         self.libA, self.libB = libA, libB
         self.kwargs["eval_clash"] = False
 
-
     def create_ensembles(self):
 
-        self.RL1 = chilife.RotamerEnsemble(self.res,
+        self.RE1 = chilife.RotamerEnsemble(self.res,
                                            self.site1,
                                            self.protein,
                                            self.chain,
                                            self.libA,
                                            **self.kwargs)
 
-        self.RL2 = chilife.RotamerEnsemble(self.res,
+        self.RE2 = chilife.RotamerEnsemble(self.res,
                                            self.site2,
                                            self.protein,
                                            self.chain,
                                            self.libB,
                                            **self.kwargs)
 
-
     def save_pdb(self, name=None):
         if name is None:
             name = self.name + ".pdb"
         if not name.endswith(".pdb"):
             name += ".pdb"
 
-        chilife.save(name, self.RL1, self.RL2)
+        chilife.save(name, self.RE1, self.RE2)
 
-    def minimize(self):
+    def minimize(self, callback=None):
 
-        scores =  [self._min_one(i, ic1, ic2) for i, (ic1, ic2) in
-                 enumerate(zip(self.RL1.internal_coords, self.RL2.internal_coords))]
+        scores = [self._min_one(i, ic1, ic2, callback=callback) for i, (ic1, ic2) in
+                  enumerate(zip(self.RE1.internal_coords, self.RE2.internal_coords))]
 
         scores = np.asarray(scores)
-        SSEs = np.linalg.norm(self.RL1.coords[:, self.cst_idx1] - self.RL2.coords[:, self.cst_idx2], axis=2).sum(axis=1)
-        MSD = SSEs/len(self.csts)
-        MSDmin= MSD.min()
+        SSEs = np.linalg.norm(self.RE1.coords[:, self.cst_idx1] - self.RE2.coords[:, self.cst_idx2], axis=2).sum(axis=1)
+        MSD = SSEs / len(self.csts)
+        MSDmin = MSD.min()
 
         if MSDmin > 0.1:
             warnings.warn(f'The minimum MSD of the cap is {MSD.min()}, this may result in distorted spin label. '
                           f'Check that the structures make sense.')
 
         if MSDmin > 0.25:
-
-           raise RuntimeError(f'chiLife was unable to connect residues {self.site1} and {self.site2} with {self.res}. '
-                              f'Please double check that this is the intended labeling site1. It is likely that these '
-                              f'sites are too far apart.')
-
-        self.RL1.backbone_to_site()
-        self.RL2.backbone_to_site()
+            raise RuntimeError(f'chiLife was unable to connect residues {self.site1} and {self.site2} with {self.res}. '
+                               f'Please double check that this is the intended labeling site. It is likely that these '
+                               f'sites are too far apart.')
+        self.cap_MSDs = MSD
+        self.RE1.backbone_to_site()
+        self.RE2.backbone_to_site()
 
         scores -= scores.min()
-
+        self.rotamer_scores = scores
         self.weights *= np.exp(-scores / (chilife.GAS_CONST * self.temp) / np.exp(-scores).sum())
         self.weights /= self.weights.sum()
 
     def _objective(self, dihedrals, ic1, ic2):
 
-        ic1.set_dihedral(dihedrals[: len(self.RL1.dihedral_atoms)], 1, self.RL1.dihedral_atoms)
-        coords1 = ic1.to_cartesian()[self.RL1.ic_mask]
+        ic1.set_dihedral(dihedrals[: len(self.RE1.dihedral_atoms)], 1, self.RE1.dihedral_atoms)
+        coords1 = ic1.to_cartesian()[self.RE1.ic_mask]
 
-        ic2.set_dihedral(dihedrals[-len(self.RL2.dihedral_atoms):], 1, self.RL2.dihedral_atoms)
-        coords2 = ic2.to_cartesian()[self.RL2.ic_mask]
+        ic2.set_dihedral(dihedrals[-len(self.RE2.dihedral_atoms):], 1, self.RE2.dihedral_atoms)
+        coords2 = ic2.to_cartesian()[self.RE2.ic_mask]
 
         diff = np.linalg.norm(coords1[self.cst_idx1] - coords2[self.cst_idx2], axis=1)
         ovlp = (coords1[self.cst_idx1] + coords2[self.cst_idx2]) / 2
@@ -264,7 +284,7 @@ class dRotamerEnsemble:
 
         # Faster to compute lj here
         lj = self.irmin_ij / r
-        lj = lj * lj *lj
+        lj = lj * lj * lj
         lj = lj * lj
 
         # attractive forces are needed, otherwise this term will perpetually push atoms apart
@@ -273,36 +293,42 @@ class dRotamerEnsemble:
 
         return score
 
-    def _min_one(self, i, ic1, ic2):
+    def _min_one(self, i, ic1, ic2, callback=None):
+        if callback is not None:
+            if 'i' in inspect.signature(callback).parameters:
+                callback = partial(callback, i=i)
 
-        d0 = np.concatenate([ic1.get_dihedral(1, self.RL1.dihedral_atoms),
-                             ic2.get_dihedral(1, self.RL2.dihedral_atoms)])
 
-        lb = [-np.pi] * len(d0)  # d0 - np.deg2rad(40)
-        ub = [np.pi] * len(d0)  # d0 + np.deg2rad(40)  #
+        d0 = np.concatenate([ic1.get_dihedral(1, self.RE1.dihedral_atoms),
+                             ic2.get_dihedral(1, self.RE2.dihedral_atoms)])
+
+        lb = d0 - np.pi  # np.deg2rad(40)
+        ub = d0 + np.pi  # np.deg2rad(40) #
         bounds = np.c_[lb, ub]
-        xopt = opt.minimize(self._objective, x0=d0, args=(ic1, ic2), bounds=bounds, method=self.min_method)
-        self.RL1._coords[i] = ic1.coords[self.RL1.H_mask]
-        self.RL2._coords[i] = ic2.coords[self.RL2.H_mask]
+        xopt = opt.minimize(self._objective, x0=d0, args=(ic1, ic2),
+                            bounds=bounds, method=self.min_method,
+                            callback=callback)
+        self.RE1._coords[i] = ic1.coords[self.RE1.H_mask]
+        self.RE2._coords[i] = ic2.coords[self.RE2.H_mask]
         tors = d0 - xopt.x
         tors = np.arctan2(np.sin(tors), np.cos(tors))
         tors = np.sqrt(tors @ tors)
 
-        return xopt.fun + tors
+        return xopt.fun + tors * self.torsion_weight
 
     @property
     def weights(self):
-        return self.RL1.weights
+        return self.RE1.weights
 
     @weights.setter
     def weights(self, value):
-        self.RL1.weights = value
-        self.RL2.weights = value
+        self.RE1.weights = value
+        self.RE2.weights = value
 
     @property
     def coords(self):
-        ovlp = (self.RL1.coords[:, self.cst_idx1] + self.RL2.coords[:, self.cst_idx2]) / 2
-        return np.concatenate([self.RL1._coords[:, self.rl1mask], self.RL2._coords[:, self.rl2mask], ovlp], axis=1)
+        ovlp = (self.RE1.coords[:, self.cst_idx1] + self.RE2.coords[:, self.cst_idx2]) / 2
+        return np.concatenate([self.RE1._coords[:, self.rl1mask], self.RE2._coords[:, self.rl2mask], ovlp], axis=1)
 
     @coords.setter
     def coords(self, value):
@@ -311,29 +337,28 @@ class dRotamerEnsemble:
                 f"The provided coordinates do not match the number of atoms of this ensemble ({self.res})"
             )
 
-        self.RL1._coords[:, self.rl1mask] = value[:, :len(self.rl1mask)]
-        self.RL2._coords[:, self.rl2mask] = value[:, len(self.rl1mask):len(self.rl1mask) + len(self.rl2mask)]
-        self.RL1._coords[:, self.cst_idx1] = value[:, len(self.rl1mask) + len(self.rl2mask):]
-        self.RL2._coords[:, self.cst_idx2] = value[:, len(self.rl1mask) + len(self.rl2mask):]
-
+        self.RE1._coords[:, self.rl1mask] = value[:, :len(self.rl1mask)]
+        self.RE2._coords[:, self.rl2mask] = value[:, len(self.rl1mask):len(self.rl1mask) + len(self.rl2mask)]
+        self.RE1._coords[:, self.cst_idx1] = value[:, len(self.rl1mask) + len(self.rl2mask):]
+        self.RE2._coords[:, self.cst_idx2] = value[:, len(self.rl1mask) + len(self.rl2mask):]
 
     @property
     def _lib_coords(self):
-        ovlp = (self.RL1._lib_coords[:, self.cst_idx1] + self.RL2._lib_coords[:, self.cst_idx2]) / 2
-        return np.concatenate([self.RL1._lib_coords[:, self.rl1mask],
-                               self.RL2._lib_coords[:, self.rl2mask], ovlp], axis=1)
+        ovlp = (self.RE1._lib_coords[:, self.cst_idx1] + self.RE2._lib_coords[:, self.cst_idx2]) / 2
+        return np.concatenate([self.RE1._lib_coords[:, self.rl1mask],
+                               self.RE2._lib_coords[:, self.rl2mask], ovlp], axis=1)
 
     @property
     def atom_names(self):
-        return np.concatenate((self.RL1.atom_names[self.rl1mask],
-                               self.RL2.atom_names[self.rl2mask],
-                               self.RL1.atom_names[self.cst_idx1]))
+        return np.concatenate((self.RE1.atom_names[self.rl1mask],
+                               self.RE2.atom_names[self.rl2mask],
+                               self.RE1.atom_names[self.cst_idx1]))
 
     @property
     def atom_types(self):
-        return np.concatenate((self.RL1.atom_types[self.rl1mask],
-                               self.RL2.atom_types[self.rl2mask],
-                               self.RL1.atom_types[self.cst_idx2]))
+        return np.concatenate((self.RE1.atom_types[self.rl1mask],
+                               self.RE2.atom_types[self.rl2mask],
+                               self.RE1.atom_types[self.cst_idx2]))
 
     @property
     def centroid(self):
@@ -367,42 +392,37 @@ class dRotamerEnsemble:
 
     @property
     def side_chain_idx(self):
-        side_chain_idx = np.argwhere(
-            np.isin(self.atom_names, dRotamerEnsemble.backbone_atoms, invert=True)
-        ).flatten()
-        return side_chain_idx
+        "Indices of side chain atoms"
+        if not hasattr(self, '_side_chain_idx'):
+            self._side_chain_idx = np.argwhere(
+                np.isin(self.atom_names, dRotamerEnsemble.backbone_atoms, invert=True)
+            ).flatten()
 
-    @property
-    def rmin2(self):
-        return np.concatenate([self.RL1.rmin2, self.RL2.rmin2])
-
-    @property
-    def eps(self):
-        return np.concatenate([self.RL1.eps, self.RL2.eps])
+        return self._side_chain_idx
 
     @property
     def bonds(self):
-        """ """
+        """List of intra-label atom pairs that are covalently bonded."""
         if not hasattr(self, "_bonds"):
             bonds = []
 
-            for bond in self.SL1.bonds:
+            for bond in self.RE1.bonds:
                 bndin = np.isin(bond, self.rl1mask)
                 if np.all(bndin):
                     bonds.append(bond)
                 elif np.any(bndin):
-                    bonds.append([bond[0], np.argwhere(self.atom_names == self.RL1.atom_names[bond[1]]).flat[0]])
+                    bonds.append([bond[0], np.argwhere(self.atom_names == self.RE1.atom_names[bond[1]]).flat[0]])
                 else:
-                    bonds.append([np.argwhere(self.atom_names == self.RL1.atom_names[bond[0]]).flat[0],
-                                 np.argwhere(self.atom_names == self.RL1.atom_names[bond[1]]).flat[0]])
+                    bonds.append([np.argwhere(self.atom_names == self.RE1.atom_names[bond[0]]).flat[0],
+                                  np.argwhere(self.atom_names == self.RE1.atom_names[bond[1]]).flat[0]])
 
-            for bond in self.SL2.bonds:
+            for bond in self.RE2.bonds:
                 bndin = np.isin(bond, self.rl2mask)
                 if np.all(bndin):
                     bonds.append([b + len(self.rl1mask) for b in bond])
                 elif not bndin[1]:
                     bonds.append([bond[0] + len(self.rl1mask),
-                                  np.argwhere(self.atom_names == self.RL2.atom_names[bond[1]]).flat[0]])
+                                  np.argwhere(self.atom_names == self.RE2.atom_names[bond[1]]).flat[0]])
 
             self._bonds = np.array(sorted(set(map(tuple, bonds))), dtype=int)
 
@@ -411,15 +431,11 @@ class dRotamerEnsemble:
     @bonds.setter
     def bonds(self, inp):
         """
-
+        Set of intralabel bonded pairs.
         Parameters
         ----------
-        inp :
-
-
-        Returns
-        -------
-
+        inp : ArrayLike
+            List of atom ID pairs that are bonded
         """
         self._bonds = set(tuple(i) for i in inp)
         idxs = np.arange(len(self.atom_names))
@@ -428,7 +444,8 @@ class dRotamerEnsemble:
 
     @property
     def non_bonded(self):
-        """ """
+        """ list of indices of intra-label non-bonded atom pairs. Primarily used for internal clash evaluation when
+        sampling the dihedral space"""
         if not hasattr(self, "_non_bonded"):
             pairs = dict(nx.all_pairs_shortest_path(self._graph, self._exclude_nb_interactions - 1))
             pairs = {(a, b) for a in pairs for b in pairs[a] if a < b}
@@ -440,15 +457,11 @@ class dRotamerEnsemble:
     @non_bonded.setter
     def non_bonded(self, inp):
         """
-
+        Create a set of non-bonded atom pair indices within a single side chain
         Parameters
         ----------
-        inp :
-
-
-        Returns
-        -------
-
+        inp : ArrayLike
+            List of atom ID pairs that are not bonded
         """
         self._non_bonded = set(tuple(i) for i in inp)
         idxs = np.arange(len(self.atom_names))
@@ -456,59 +469,59 @@ class dRotamerEnsemble:
         self._bonds = all_pairs - self._non_bonded
 
     def trim_rotamers(self):
-        self.RL1.trim_rotamers()
-        self.RL2.trim_rotamers()
+        """Remove low probability rotamers from the ensemble"""
+
+        arg_sort_weights = np.argsort(self.weights)[::-1]
+        sorted_weights = self.weights[arg_sort_weights]
+        cumulative_weights = np.cumsum(sorted_weights)
+        cutoff = np.maximum(1, len(cumulative_weights[cumulative_weights < 1 - self.trim_tol]))
+        keep_idx = arg_sort_weights[:cutoff]
+        if hasattr(self, 'cap_MSDs'):
+            self.cap_MSDs = self.cap_MSDs[keep_idx]
+            self.rotamer_scores = self.rotamer_scores[keep_idx]
+
+        if hasattr(self, 'rot_clash_energy'):
+            self.rot_clash_energy = self.rot_clash_energy[keep_idx]
+        self.RE1.trim_rotamers(keep_idx=keep_idx)
+        self.RE2.trim_rotamers(keep_idx=keep_idx)
 
     def evaluate(self):
         """Place rotamer ensemble on protein site1 and recalculate rotamer weights."""
         # Calculate external energies
         energies = self.energy_func(self)
-
+        self.rot_clash_energy = energies
         # Calculate total weights (combining internal and external)
         self.weights, self.partition = chilife.reweight_rotamers(energies, self.temp, self.weights)
         logging.info(f"Relative partition function: {self.partition:.3}")
 
         # Remove low-weight rotamers from ensemble
-        self.trim_rotamers()
+        if self._do_trim:
+            self.trim_rotamers()
 
     def __len__(self):
-        return len(self.weights)
+        """Return the length of the rotamer library (number of rotamers)"""
+        return len(self.RE1.coords)
 
+    def copy(self):
+        """Create a deep copy of the dRotamerEnsemble object"""
+        new_copy = chilife.dRotamerEnsemble(self.res, (self.site1, self.site2), chain=self.chain,
+                                            protein=self.protein,
+                                            rotlib={'csts': self.csts, 'libA': self.libA, 'libB': self.libB},
+                                            minimize=False,
+                                            eval_clash=False)
+        for item in self.__dict__:
+            if isinstance(self.dict[item], np.ndarray):
+                new_copy.__dict__[item] = self.__dict__[item].copy()
 
-def get_possible_rotlibs(rotlib: str, all: bool = False) -> Union[Path, None]:
-    """
+            elif item == 'RE1':
+                new_copy.__dict__[item] == self.__dict__[item].copy(rotlib=self.libA)
 
-    """
-    cwd = Path.cwd()
+            elif item == 'RE2':
+                new_copy.__dict__[item] == self.__dict__[item].copy(rotlib=self.libB)
 
-    # Assemble a list of possible rotlib paths starting in the current directory
-    possible_rotlibs = [Path(rotlib),
-                        cwd / rotlib,
-                        cwd / (rotlib + '.zip'),
-                        cwd / (rotlib + '_drotlib.zip')]
+            elif item == 'protein':
+                pass
 
-    possible_rotlibs += list(cwd.glob(f'{rotlib}*_drotlib.zip'))
-    # Then in the user defined rotamer library directory
-    for pth in chilife.USER_RL_DIR:
-        possible_rotlibs += list(pth.glob(f'{rotlib}*_drotlib.zip'))
-
-    # Then in the chilife directory
-    possible_rotlibs += list((chilife.RL_DIR / 'user_rotlibs').glob(f'*{rotlib}*'))
-
-    if all:
-        rotlib = []
-    for possible_file in possible_rotlibs:
-        if possible_file.exists() and all:
-                rotlib.append(possible_file)
-        elif possible_file.exists():
-            rotlib = possible_file
-            break
-    else:
-        if not isinstance(rotlib, list) or rotlib == []:
-            rotlib = None
-
-    # rotlib lists need to be sorted to prevent position mismatches for results with tests.
-    if isinstance(rotlib, list):
-        rotlib = sorted(rotlib)
-
-    return rotlib
+            else:
+                new_copy.__dict__[item] = deepcopy(self.__dict__[item])
+        return new_copy
