@@ -9,13 +9,14 @@ import zipfile
 
 import numpy as np
 from scipy.stats import gaussian_kde
-from memoization import cached
+from memoization import cached, suppress_warnings
 import MDAnalysis as mda
 
 import chilife
 from .RotamerEnsemble import RotamerEnsemble
 from .SpinLabel import SpinLabel
 from .dRotamerEnsemble import dRotamerEnsemble
+from .IntrinsicLabel import IntrinsicLabel
 from .Protein import MolecularSystem
 from .ProteinIC import ProteinIC
 
@@ -62,7 +63,7 @@ def hash_file(file: Union[Path, BinaryIO]):
     return hash.hexdigest()
 
 
-#
+suppress_warnings()
 @cached(custom_key_maker=hash_file)
 def read_rotlib(rotlib: Union[Path, BinaryIO] = None) -> Dict:
     """Reads RotamerEnsemble for stored spin labels.
@@ -263,7 +264,7 @@ def save(
         Object(s) to save. Includes RotamerEnsemble, SpinLabels, dRotamerEnsembles, proteins, or path to protein pdb.
         Can add as many as desired, except for path, for which only one can be given.
     protein_path : str, Path
-        Path to a pdb file to use as the protein object.
+        The Path to a pdb file to use as the protein object.
     **kwargs :
         Additional arguments to pass to ``write_labels``
 
@@ -290,7 +291,6 @@ def save(
         tmolecules[mcls[0]].append(mol)
     molecules = tmolecules
 
-
     # Ensure only one protein path was provided (for now)
     protein_path = [] if protein_path is None else [protein_path]
     if len(protein_path) > 1:
@@ -401,17 +401,14 @@ def write_protein(pdb_file: TextIO, protein: Union[mda.Universe, mda.AtomGroup])
 
 def write_ic(pdbfile: TextIO, ic: chilife.ProteinIC) -> None:
     """
-    Write a chilife.ProteinIC internal coordinates object to a pdb file.
-      Parameters
+    Write a :class:`chilife.ProteinIC` internal coordinates object to a pdb file.
+
+    Parameters
     ----------
-    file : TextIO
+    pdbfile : TextIO
         open file or io object.
     ic: chilife.ProteinIC
         chiLife internal coordinates object.
-
-    Returns
-    -------
-    None
     """
     pdbfile.write('MODEL\n')
     for atom, coord in zip(ic.atoms, ic.coords):
@@ -450,14 +447,14 @@ def write_labels(pdb_file: TextIO, *args: SpinLabel, KDE: bool = True, sorted: b
         sorted_index = np.argsort(label.weights)[::-1] if sorted else np.arange(len(label.weights))
         norm_weights = label.weights / label.weights.max()
         if isinstance(label, dRotamerEnsemble):
-            sites = np.concatenate([np.ones(len(label.RL1.atoms), dtype=int) * int(label.site1),
-                                    np.ones(len(label.RL2.atoms), dtype=int) * int(label.site2)])
+            sites = np.concatenate([np.ones(len(label.RE1.atoms), dtype=int) * int(label.site1),
+                                    np.ones(len(label.RE2.atoms), dtype=int) * int(label.site2)])
         else:
-            sites = np.ones(len(label.atoms), dtype=int) * int(label.site)
+            sites = [atom.resi for atom in label.atoms]
         for mdl, (conformer, weight) in enumerate(
                 zip(label.coords[sorted_index], norm_weights[sorted_index])
         ):
-            pdb_file.write("MODEL {}\n".format(mdl))
+            pdb_file.write(f"MODEL {mdl}\n")
 
             [
                 pdb_file.write(
@@ -485,7 +482,7 @@ def write_labels(pdb_file: TextIO, *args: SpinLabel, KDE: bool = True, sorted: b
         if not np.any(label.spin_centers):
             continue
 
-        pdb_file.write(f"HEADER {label.name}_density\n".format(label.label, k + 1))
+        pdb_file.write(f"HEADER {label.name}_density\n")
         spin_centers = np.atleast_2d(label.spin_centers)
 
         if KDE and len(spin_centers) > 5:
@@ -519,6 +516,7 @@ def write_labels(pdb_file: TextIO, *args: SpinLabel, KDE: bool = True, sorted: b
 
 molecule_class = {RotamerEnsemble: 'rotens',
                   dRotamerEnsemble: 'rotens',
+                  IntrinsicLabel: 'rotens',
                   mda.Universe: 'molcart',
                   mda.AtomGroup: 'molcart',
                   MolecularSystem: 'molcart',
