@@ -8,7 +8,7 @@ import chilife
 
 
 def clash_only(func):
-    """decorator to convert a Lennard-Jones style clash evaluation function into a chiLife compatible energy func.
+    """Decorator to convert a Lennard-Jones style clash evaluation function into a chiLife compatible energy func.
 
     Parameters
     ----------
@@ -18,48 +18,48 @@ def clash_only(func):
 
     Returns
     -------
-    energy_func: callable
+    energy_func : callable
         The original input function now wrapped to accept a protein object and, optionally, a RotamerEnsemble/SpinLabel
-         object.
+        object.
     """
 
     @wraps(func)
     def energy_func(system, **kwargs):
         """
-
         Parameters
         ----------
         system : MDAnalysis.Universe, MDAnalysis.AtomGroup, RotamerEnsemble, Protein
             Molecular system for which you wish to calculate the energy of
 
-        **kwargs :
-            
+        **kwargs : dict
+            Additional arguments to pass to the wrapped function
 
         Returns
         -------
         E : numpy.ndarray
             Array of energy values (kcal/mol) for each rotamer in an ensemble or for the whole system.
-
         """
         internal = kwargs.pop("internal", False)
         rmax = kwargs.get("rmax", 10)
         forgive = kwargs.get("forgive", 1)
         _protein = kwargs.pop('protein', None)
 
-        if isinstance(system, (chilife.RotamerEnsemble, chilife.dSpinLabel)):
+        if isinstance(system, (chilife.RotamerEnsemble, chilife.dRotamerEnsemble)):
             if system.protein is None and _protein is not None:
                 system.protein = _protein
 
-            r, rmin, eps, shape = prep_external_clash(system)
-            E = func(r, rmin, eps, **kwargs).reshape(shape)
+            r, rmin, eps = prep_external_clash(system)
+            E = func(r, rmin, eps, **kwargs)
 
             if internal:
                 r, rmin, eps, shape = prep_internal_clash(system)
                 Einternal = func(r, rmin, eps, **kwargs)
                 E = np.concatenate((E, Einternal), axis=1)
 
+            system.atom_energies = E.reshape(len(E), len(system.side_chain_idx), -1).sum(axis=2)
             E = E.sum(axis=1)
 
+
         elif isinstance(system, (mda.Universe, mda.AtomGroup, chilife.Protein)):
             bonds = {(a, b) for a, b in system.atoms.bonds.indices}
             tree = cKDTree(system.atoms.positions)
@@ -128,24 +128,26 @@ def get_lj_energy(r, rmin, eps, forgive=1, cap=10, rmax=10):
         Vector of atom pair energies calculated using the modified lj potential function.
 
     """
+    r = np.atleast_2d(r)
     lj_energy = np.zeros_like(r)
     rmin_lower = forgive * rmin
 
     # Piecewise function for flat lj potential near rmin
-    for i in range(len(r)):
-        if r[i] < rmin_lower[i]:
-            lj = rmin_lower[i] / r[i]
-            lj = lj * lj * lj
-            lj = lj * lj
-            lj_energy[i] = np.minimum(eps[i] * (lj**2 - 2 * lj), cap * eps[i])
+    for j, rotamer in enumerate(r):
+        for i in range(len(rotamer)):
+            if rotamer[i] < rmin_lower[i]:
+                lj = rmin_lower[i] / rotamer[i]
+                lj = lj * lj * lj
+                lj = lj * lj
+                lj_energy[j, i] = np.minimum(eps[i] * (lj**2 - 2 * lj), cap * eps[i])
 
-        elif rmin_lower[i] <= r[i] < rmin[i]:
-            lj_energy[i] = -eps[i]
-        elif r[i] < rmax:
-            lj = rmin[i] / r[i]
-            lj = lj * lj * lj
-            lj = lj * lj
-            lj_energy[i] = eps[i] * (lj**2 - 2 * lj)
+            elif rmin_lower[i] <= rotamer[i] < rmin[i]:
+                lj_energy[j, i] = -eps[i]
+            elif rotamer[i] < rmax:
+                lj = rmin[i] / rotamer[i]
+                lj = lj * lj * lj
+                lj = lj * lj
+                lj_energy[j, i] = eps[i] * (lj**2 - 2 * lj)
 
     return lj_energy
 
@@ -175,22 +177,23 @@ def get_lj_scwrl(r, rmin, eps, forgive=1):
     lj_energy: numpy.ndarray
         Vector of atom pair energies calculated using the modified lj potential function.
     """
+    r = np.atleast_2d(r)
     lj_energy = np.empty_like(r)
     rmin_lower = rmin * forgive
-
-    # Piecewise function for flat lj potential near rmin
-    for i in range(len(r)):
-        rat = r[i] / (rmin_lower[i] / 1.12246204831)
-        if rat < 0.8254:
-            lj_energy[i] = 10 * eps[i]
-        elif rat <= 1:
-            lj_energy[i] = 57.273 * (1 - rat) * eps[i]
-        elif rat < 10 / 9:
-            lj_energy[i] = eps[i] * (10 - 9 * rat) ** (57.273 / (9 * eps[i])) - eps[i]
-        elif rat < 4 / 3:
-            lj_energy[i] = (eps[i] / 4) * (9 * rat - 10) ** 2 - eps[i]
-        else:
-            lj_energy[i] = 0
+    for j, rotamer in enumerate(r):
+        # Piecewise function for flat lj potential near rmin
+        for i in range(len(rotamer)):
+            rat = rotamer[i] / (rmin_lower[i] / 1.12246204831)
+            if rat < 0.8254:
+                lj_energy[j, i] = 10 * eps[i]
+            elif rat <= 1:
+                lj_energy[j, i] = 57.273 * (1 - rat) * eps[i]
+            elif rat < 10 / 9:
+                lj_energy[j, i] = eps[i] * (10 - 9 * rat) ** (57.273 / (9 * eps[i])) - eps[i]
+            elif rat < 4 / 3:
+                lj_energy[j, i] = (eps[i] / 4) * (9 * rat - 10) ** 2 - eps[i]
+            else:
+                lj_energy[j, i] = 0
 
     return lj_energy
 
@@ -224,15 +227,17 @@ def get_lj_rep(r, rmin, eps, forgive=0.9, cap=10):
     lj_energy: numpy.ndarray
         Vector of atom pair energies calculated using the modified lj potential function.
     """
+    r = np.atleast_2d(r)
     lj_energy = np.empty_like(r)
     rmin_lower = forgive * rmin
 
     # Piecewise function for flat lj potential near rmin
-    for i in range(len(r)):
-        lj = rmin_lower[i] / r[i]
-        lj = lj * lj * lj
-        lj = lj * lj
-        lj_energy[i] = np.minimum(eps[i] * lj**2, cap * eps[i])
+    for i, rotamer in enumerate(r):
+        for j in range(len(rotamer)):
+            lj = rmin_lower[j] / rotamer[j]
+            lj = lj * lj * lj
+            lj = lj * lj
+            lj_energy[i, j] = np.minimum(eps[j] * lj**2, cap * eps[j])
 
     return lj_energy
 
@@ -265,15 +270,16 @@ def get_lj_attr(r, rmin, eps, forgive=0.9, floor=-2):
       lj_energy: numpy.ndarray
           Vector of atom pair energies calculated using the modified lj potential function.
       """
+    r = np.atleast_2d([r])
     lj_energy = np.empty_like(r)
     rmin_lower = forgive * rmin
-
-    # Piecewise function for flat lj potential near rmin
-    for i in range(len(r)):
-        lj = rmin_lower[i] / r[i]
-        lj = lj * lj * lj
-        lj = lj * lj
-        lj_energy[i] = np.maximum(-2 * eps[i] * lj, eps[i] * floor)
+    for j, rotamer in enumerate(r):
+        # Piecewise function for flat lj potential near rmin
+        for i in range(len(rotamer)):
+            lj = rmin_lower[i] / rotamer[i]
+            lj = lj * lj * lj
+            lj = lj * lj
+            lj_energy[j, i] = np.maximum(-2 * eps[i] * lj, eps[i] * floor)
 
     return lj_energy
 
@@ -299,19 +305,22 @@ def prep_external_clash(ensemble):
     """
 
     # Calculate rmin and epsilon for all atoms in protein that may clash
-    rmin_ij, eps_ij = get_lj_params(ensemble)
+    if hasattr(ensemble, 'ermin_ij'):
+        rmin_ij = ensemble.ermin_ij
+        eps_ij = ensemble.eeps_ij
+    else:
+        rmin_ij, eps_ij = get_lj_params(ensemble)
+        ensemble.ermin_ij = rmin_ij
+        ensemble.eeps_ij = eps_ij
 
+    if len(ensemble) == 1:
+        ec = ensemble.coords[0, ensemble.side_chain_idx]
+    else:
+        ec = ensemble.coords[:, ensemble.side_chain_idx].reshape(-1, 3)
+    pc = ensemble.protein_tree.data[ensemble.protein_clash_idx]
     # Calculate distances
-    dist = cdist(
-        ensemble.coords[:, ensemble.side_chain_idx].reshape(-1, 3),
-        ensemble.protein_tree.data[ensemble.protein_clash_idx],
-    ).ravel()
-    shape = (
-        len(ensemble.coords),
-        len(ensemble.side_chain_idx) * len(ensemble.protein_clash_idx),
-    )
-
-    return dist, rmin_ij, eps_ij, shape
+    dist = cdist(ec, pc).reshape(len(ensemble), -1)
+    return dist, rmin_ij, eps_ij
 
 
 def prep_internal_clash(ensemble):
@@ -334,19 +343,27 @@ def prep_internal_clash(ensemble):
                 Shape the array should be so that the energy is evaluated for each rotamer of the ensemble separately.
         """
 
-    a, b = [list(x) for x in zip(*ensemble.non_bonded)]
-    a_eps = chilife.get_lj_eps(ensemble.atom_types[a])
-    a_radii = chilife.get_lj_rmin(ensemble.atom_types[a])
-    b_eps = chilife.get_lj_eps(ensemble.atom_types[b])
-    b_radii = chilife.get_lj_rmin(ensemble.atom_types[b])
+    if hasattr(ensemble, 'ermin_ij'):
+        rmin_ij, eps_ij = ensemble.ermin_ij, ensemble.eeps_ij
+    else:
+        a, b = [list(x) for x in zip(*ensemble.non_bonded)]
+        a_eps = chilife.get_lj_eps(ensemble.atom_types[a])
+        a_radii = chilife.get_lj_rmin(ensemble.atom_types[a])
+        b_eps = chilife.get_lj_eps(ensemble.atom_types[b])
+        b_radii = chilife.get_lj_rmin(ensemble.atom_types[b])
 
-    join_rmin = chilife.get_lj_rmin("join_protocol")[()]
-    join_eps = chilife.get_lj_eps("join_protocol")[()]
+        join_rmin = chilife.get_lj_rmin("join_protocol")[()]
+        join_eps = chilife.get_lj_eps("join_protocol")[()]
 
-    rmin_ij = join_rmin(a_radii * ensemble.forgive, b_radii * ensemble.forgive, flat=True)
-    eps_ij = join_eps(a_eps, b_eps, flat=True)
+        rmin_ij = join_rmin(a_radii * ensemble.forgive, b_radii * ensemble.forgive, flat=True)
+        eps_ij = join_eps(a_eps, b_eps, flat=True)
 
-    dist = np.linalg.norm(ensemble.coords[:, a] - ensemble.coords[:, b], axis=2)
+        ensemble.irmin_ij = rmin_ij
+        ensemble.ieps_ij = eps_ij
+        ensemble.aidx = a
+        ensemble.bidx = b
+
+    dist = np.linalg.norm(ensemble.coords[:, ensemble.aidx] - ensemble.coords[:, ensemble.bidx], axis=2)
     shape = (len(ensemble.coords), len(a_radii))
 
     return dist, rmin_ij, eps_ij, shape
@@ -407,15 +424,8 @@ def get_lj_params(ensemble):
     join_rmin = chilife.get_lj_rmin("join_protocol")[()]
     join_eps = chilife.get_lj_eps("join_protocol")[()]
 
-    rmin_ij = np.tile(
-        join_rmin(
-            ensemble.rmin2 * ensemble.forgive, protein_lj_radii * ensemble.forgive
-        ).reshape(-1),
-        len(ensemble.coords),
-    )
-    eps_ij = np.tile(
-        join_eps(ensemble.eps, protein_lj_eps).reshape((-1)), len(ensemble.coords)
-    )
+    rmin_ij = join_rmin(ensemble.rmin2 * ensemble.forgive, protein_lj_radii * ensemble.forgive).reshape(-1)
+    eps_ij = join_eps(ensemble.eps, protein_lj_eps).reshape((-1))
 
     return rmin_ij, eps_ij
 
@@ -475,7 +485,7 @@ rmin_charmm = {
      "N": 1.8500,
      "O": 1.7000,
      "S": 2.0000,
-    "SE": 2.0000,  # Default Selenium to Sulfur
+    "Se": 2.0000,  # Default Selenium to Sulfur
     "Br": 1.9800,
     "Cu": 0.7251,  # ion 2+
      'B': 2.5500,
@@ -499,7 +509,7 @@ eps_charmm = {
     "N": -0.200,
     "O": -0.120,
     "S": -0.450,
-    "SE": -0.450,
+    "Se": -0.450,
     "Br": -0.320,
     "Cu": -0.1505,
      'B': -0.0380,
@@ -523,8 +533,20 @@ rmin_uff = {
     "N": 3.660,
     "O": 3.500,
     "S": 4.035,
-    "SE": 4.035,  # Default Selenium to Sulfur
+    "Se": 4.035,  # Default Selenium to Sulfur
     "Br": 4.189,
+    "Na": 2.983,
+    "Mg": 3.021,
+    "P" : 4.147,
+    "Cl": 3.947,
+    "K" : 3.812,
+    "Ca": 3.399,
+    "Mn": 2.961,
+    "Fe": 2.912,
+    "Co": 2.872,
+    "Ni": 2.834,
+    "Cu": 3.495,
+    "Gd": 3.368,
     "join_protocol": join_geom,
 }
 
@@ -534,11 +556,28 @@ eps_uff = {
     "N": -0.069,
     "O": -0.060,
     "S": -0.274,
-    "SE": -0.274,  # Default Selenium to Sulfur
+    "Se": -0.274,  # Default Selenium to Sulfur
     "Br": -0.251,
+    "Na": -0.030,
+    "Mg": -0.111,
+    "P" : -0.305,
+    "Cl": -0.227,
+    "K" : -0.035,
+    "Ca": -0.238,
+    "Mn": -0.013,
+    "Fe": -0.013,
+    "Co": -0.014,
+    "Ni": -0.015,
+    "Cu": -0.005,
+    "Gd": -0.009,
     "join_protocol": join_geom,
 }
 
+for dictionary in rmin_charmm, eps_charmm, rmin_uff, eps_uff:
+    tdict = {**{key.upper(): val for key, val in dictionary.items()},
+             **{key.lower(): val for key, val in dictionary.items()}}
+    dictionary.update(tdict)
+
 lj_params = {"uff": [rmin_uff, eps_uff], "charmm": [rmin_charmm, eps_charmm]}
 
 
